#include <avr/io.h>
#include <avr/interrupt.h>

#include <util/delay.h>

#include <stdlib.h>
#include <stdio.h>

/* definitii note (in hertzi) */
#define c    261
#define d    294
#define e    329
#define f    349
#define g    391
#define gS   415
#define a    440
#define aS   455
#define b    466
#define cH   523
#define cSH  554
#define dH   587
#define dSH  622
#define eH   659
#define fH   698
#define fSH  740
#define gH   784
#define gSH  830
#define aH   880
uint16_t imperial_notes[] = {a, a, a, f, cH, a, f, cH, a, eH, eH, eH,
 fH, cH, gS, f, cH, a, aH, a, a, aH, gSH, gH, fSH, fH, fSH, aS, dSH,
 dH, cSH, cH, b, cH, f, gS, f, a, cH, a, cH, eH, aH, a, a, aH, gSH,
 gH, fSH, fH, fSH, aS, dSH, dH, cSH, cH, b, cH, f, gS, f, cH, a, f, cH, a};

uint16_t imperial_durations[] = {500, 500, 500, 350, 150, 500, 350, 150,
 650, 500, 500, 500, 350, 150, 500, 350, 150, 650, 500, 300, 150, 400, 200, 200,
 125, 125, 250, 250, 400, 200, 200, 125, 125, 250, 125, 500, 375, 125, 500, 375,
 125, 650, 500, 300, 150, 400, 200, 200, 125, 125, 250, 250, 400, 200, 200, 125,
 125, 250, 250, 500, 375, 125, 500, 375, 125, 650};

 
uint16_t bells_notes[] = {e,e,e,e,e,e,e,g,c,d,e,f,f,f,
f,f,e,e,e,e,d,d,e,d,g};
uint16_t bells_durations[] = {1, 1, 2, 1, 1, 2, 1, 1, 1, 1, 2,
 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2};
uint8_t num_imperial = sizeof(imperial_durations) / sizeof(imperial_durations[0]);
uint8_t num_bells = sizeof(bells_notes) / sizeof(bells_notes[0]);

int song = 1;
void Speaker_play_stop()
{
    DDRD ^= (1 << PD5);
}


void timer1_init_ex6()
{
    /* timer1 este configurat cu modul CTC, top OCR1A si prescaler 1024 */
    TCCR1A |= (1 << COM1A0) | (1 << COM1B0);
    TCCR1B |= (5 << CS10) | (1 << WGM12);

    OCR1B = 0;
    OCR1A = 0;
}

//int0



int PLAY = 1 ; 
ISR(INT0_vect){
	PORTD ^= (1 << PD7);
	Speaker_play_stop();
	PLAY ^= 1;
}
int reset = 0;
ISR(INT1_vect){
	reset=1; 
}
void update_song()
{
	if(PLAY == 0 ) 
	{
		return ;
	}
	static uint8_t note_remaining = 1;
	static int8_t note_position = -1;
	if(reset)
	{
		note_remaining = 1 ;
		note_position = -1 ;
		reset = 0;	
		song ^= 1;
	}
    if(--note_remaining == 0) //imperial march
    {
		uint16_t new_top;
		if(song == 1 ) //jingle bells 
		{
			note_position = (note_position + 1) % num_bells;
			note_remaining = bells_durations[note_position] * 17;
			new_top = 16000000 / 1024 / 2 / bells_notes[note_position];
		}
		else
		{
			note_position = (note_position + 1) % num_imperial;
			note_remaining = imperial_durations[note_position] / 25;
			new_top = 16000000 / 1024 / 2 / imperial_notes[note_position];
		}
         
		TCNT1 = 0;
        OCR1B = (uint32_t)new_top * OCR1B / OCR1A;
		OCR1A = new_top;
    }
}


int main(void)
{
    Speaker_play_stop();

    timer1_init_ex6();

    sei();
	EICRA |= (1 << ISC01) | (1 << ISC00);
	EIMSK |= (1 << INT0);
	EIMSK |= (1 << INT1);
	DDRD |= (1 << PD7);	

    while(1)
    {

		update_song();

		_delay_ms(25);
    }

    return 0;
}

